/* Tennis Tournament Management System 2005 by Silvio Moioli
    and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it).

	


	ATTENZIONE: questa è la versione DEBUG del programma, pertanto non
	contiene alcuni dei cambiamenti che sono inglogati nella versione
	RELEASE. Si consiglia di usare sempre e solo la versione RELEASE.

	Questa versione è stata pubblicata SOLO per documentare le fasi intermedie
	del lavoro con le assertions e i programmi di prova (come descritto nella
	documentazione)!!!


 */
 #include "file.h"

/**
 * Crea un nuovo file (vuoto).
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * controllare, completo di percorso.
 */
void fileCrea(const char* nomeFile){
	assert(strcmp(nomeFile,""));
	assert(!fileEsiste(nomeFile));
    ofstream daCreare;
    daCreare.open(nomeFile, ios::app);
    if(daCreare.fail()){
        cout<<"Impossibile creare il file. Disco pieno?"<<endl;
        exit(1);
    }
    daCreare.close();
}

/**
 * Legge la lunghezza (in byte) del file.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * controllare, completo di percorso.
 * @return il numero di byte nel file
 */
int fileLunghezza(const char* nomeFile){
	assert(fileEsiste(nomeFile));
	ifstream daContare;
    daContare.open(nomeFile);
    daContare.seekg(0,ios::end);
    int risultato = daContare.tellg();
    daContare.close();
    return risultato;
}

/**
 * Controlla se il prossimo campo da aggiungere &egrave; il primo del suo 
 * record.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * controllare, completo di percorso.
 * @return true se il prossimo campo aggiunto sar&agrave; il primo del suo
 * record, false altrimenti.
 */
bool primoCampo(const char* nomeFile){
	assert(fileEsiste(nomeFile));
    int lunghezza =  fileLunghezza(nomeFile);
	if (lunghezza==0){
		return true;
	}
    
    ifstream daControllare;
	daControllare.open(nomeFile);
	daControllare.seekg(-1,ios::end);
    char precedente=daControllare.get();
    daControllare.close();
	if (precedente == '\n'){
		return true;
	}
	
	return false;
}

/**
 * Conta il numero di record all'interno del file
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @return il numero di record presenti nel file
 */
int fileContaRecordInt(ifstream* file){
   int record=0;
   file->seekg(0,ios::beg);
   while(!file->eof()){
	    char c=file->get();
        if (c=='\n'){
	        record++;
        }
        assert(record<100);
   }

   //Mi tolgo dall'eof e finisco
   file->seekg(0,ios::beg);
   file->clear();
   return record;
}

/**
 * Torna il numero di byte dall'inizio del file al primo carattere dell'y-esimo
 * record in esso contenuto.
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @param y l'indice del record
 * @return il numero di byte dall'inizio del file al primo carattere 
 * dell'y-esimo record
 */
void filePosizionaRecord(ifstream* file, int y){
   assert(y>=0 && y<fileContaRecordInt(file));

   file->seekg(0,ios::beg);  
   //Salta i record
   for (int i=0; i<y; i++){
    char c='a';
    while((c!='\n')&&(!file->eof())){
         c=file->get();
    }
   }
   
   //int risultato = file->tellg();
   //Mi tolgo dall'eof (se ci sono) e finisco
/*   file->seekg(0,ios::beg);
   file->clear();
   return risultato;*/
}

/**
 * Conta il numero di campi in un record.
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @param y l'indice del record
 * @return il numero di campi nell' y-esimo record
 */
int fileContaCampi(ifstream* file, int y){
   assert(y>=0 && y<fileContaRecordInt(file));

   //file->seekg(filePosizioneRecord(file, y));
   filePosizionaRecord(file, y);
   
   int campi=0;
   char c = 'a';  
   while(c!='\n'){
        c=file->get();
        if (c==','){
	        campi++;
        }
        assert(campi<100);
   }
   //Mi tolgo dall'eof (se ci sono) e finisco
   file->seekg(0,ios::beg);
   file->clear();
   return campi +1;
}

/**
 * Torna il numero di byte dall'inizio del file al primo carattere dell'x-esimo
 * campo nell'y-esimo record in esso contenuto.
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @param x l'indice del campo
 * @param y l'indice del record
 * @return il numero di byte dall'inizio del file al primo carattere 
 * dell'x-esimo campo nell'y-esimo record
 */
void filePosizionaCampo(ifstream* file, int x, int y){
   assert(y>=0 && y<fileContaRecordInt(file));
   assert(x>=0 && x<fileContaCampi(file,y));
   
   //file->seekg(filePosizioneRecord(file, y));
   filePosizionaRecord(file, y);
   
   //Salta i campi
   for (int i=0; i<x; i++){
    char c='a';
    while(c!=','){
         c=file->get();
    }
   }

   //int risultato = file->tellg();

   //Mi tolgo dall'eof (se ci sono) e finisco
/*   file->seekg(0,ios::beg);
   file->clear();
   return risultato;*/
}

/** 
 * Legge e ritorna il numero di caratteri da questa posizione
 * del file alla prossima virgola o fine riga.
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 */
int lunghezzaProssimoCampo(ifstream* file){
	int risultato = 0;
	
	char c=file->get();
    while(c!=',' && c!='\n'){
         risultato++;
         c=file->get();
    }
	
	return risultato;
}

/**
 * Controlla l'esistenza di un file.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * controllare, completo di percorso.
 * @return true se il file esiste, false altrimenti.
 */
bool fileEsiste(const char* nomeFile){
	assert(strcmp(nomeFile,""));
	ifstream daControllare;
	daControllare.open(nomeFile);
	if (daControllare.fail()){
		return false;
	}
	else{
		daControllare.close();
		return true;
	}
}

/**
 * Aggiungere un campo in coda al file.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @param s campo da inserire anch'esso in formato stringa
 */
void fileAggiungiCampo(const char* nomeFile, const char* s)
{
    assert(strcmp(nomeFile,""));
	if (!fileEsiste(nomeFile)){
		fileCrea(nomeFile);
	}
    
	bool primo = primoCampo(nomeFile);
	
	ofstream fileScrivi;
	fileScrivi.open(nomeFile, ios::app);
	if (!primo){
	    fileScrivi<<",";
	}
    fileScrivi<<s;
    fileScrivi.close();
}

/**
 * Aggiungere un campo in coda al file, sovrascrivendo l'ultimo campo
 * dell'ultimo record presente.
 * Suppone che l'ultimo record del file abbia un solo campo.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @param s campo da inserire anch'esso in formato stringa
 */
void fileSovrascriviUltimoCampo(const char* nomeFile, const char* s)
{
    assert(strcmp(nomeFile,""));
	if (!fileEsiste(nomeFile)){
		fileCrea(nomeFile);
		fileAggiungiCampo(nomeFile,s);
	}
	else{
		filebuf fileScrivi;
	    ostream output(&fileScrivi);
	    istream input(&fileScrivi);
	    fileScrivi.open (nomeFile, ios::in | ios::out);
   
		output.seekp(-1, ios_base::end);
		output<<s;
		fileScrivi.close();
	}
}

/**
 * Chiude il record corrente. La prossima operazione di fileAggiungiCampo
 * avverr&agrave; su un nuovo record.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 */
void fileNuovoRecord(const char* nomeFile)
{
    assert(strcmp(nomeFile,""));
	if (!fileEsiste(nomeFile)){
		fileCrea(nomeFile);
	}
	
	ofstream fileScrivi;
	fileScrivi.open(nomeFile, ios::app);
	fileScrivi<<endl;
    fileScrivi.close();
}

/**
 * Legge e ritorna l'x-esimo campo nell'y-esimo record del file in formato
 * stringa.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @param x l'indice del campo
 * @param y l'indice del record
 * @return una stringa appena allocata contenente il dato letto
 */
char* fileLeggiCampo(const char* nomeFile, int x, int y){
    assert(strcmp(nomeFile,""));
    assert(fileEsiste(nomeFile));
    ifstream file;
    file.open(nomeFile);    
    assert(y>=0 && y<fileContaRecordInt(&file));
    assert(x>=0 && x<fileContaCampi(&file,y));
   
    //file.seekg(filePosizioneCampo(&file,x,y));
    filePosizionaCampo(&file,x,y);
    
    /**
     * Vecchia implementazione: inspiegabilmente ha problemi
     * su alcune versioni del gcc su Windows, in particolare
     * con quella del dev-c++ 5.
     * 
     * int puntatorePrecedente = file.tellg(); 
     * int lunghezza = lunghezzaProssimoCampo(&file);
     * assert(lunghezza<100);
     * char* risultato = new char[lunghezza+1];
     * file.seekg(puntatorePrecedente);	
     */
     
     //Nuova implementazione: meno efficiente...
     int lunghezza = lunghezzaProssimoCampo(&file);
     char* risultato = new char[lunghezza+1];
     //...perche' richiama fpc() due volte.
    filePosizionaCampo(&file,x,y);  	
   
    for(int i=0;i<lunghezza;i++){
        risultato[i]=file.get(); 
    }
    risultato[lunghezza]='\0';
    
    file.close();
    return risultato;
}

/**
 * Conta il numero di record all'interno del file.
 * Versione pubblica (non richiede l'oggetto ifstream
 * ma solo il nome del file).
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file,
 *  completo di percorso.
 * @return il numero di record presenti nel file
 */
int fileContaRecord(const char* nomeFile){
	assert(strcmp(nomeFile,""));
    assert(fileEsiste(nomeFile));
    ifstream file;
    file.open(nomeFile);    
	int risultato = fileContaRecordInt(&file);
	file.close();
	return risultato;
}

/**
 * Legge e ritorna l'ultimo carattere in un file, prima dell'eof.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file,
 *  completo di percorso.
 * @return l'ultimo carattere nel file
 */
char fileUltimoCarattere(const char* nomeFile){
	assert(strcmp(nomeFile,""));
    assert(fileEsiste(nomeFile));
    ifstream file;
    file.open(nomeFile);
    file.seekg(-1, ios::end);
	char risultato = file.get();
	file.close();
	return risultato;
}
