/* Tennis Tournament Management System 2005 by Silvio Moioli
    and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it).

	


	ATTENZIONE: questa è la versione DEBUG del programma, pertanto non
	contiene alcuni dei cambiamenti che sono inglogati nella versione
	RELEASE. Si consiglia di usare sempre e solo la versione RELEASE.

	Questa versione è stata pubblicata SOLO per documentare le fasi intermedie
	del lavoro con le assertions e i programmi di prova (come descritto nella
	documentazione)!!!


 */
#include "giocatore.h"
#include "partita.h"
#include "file.h"
#include "liste.h"

/**
 * Ritorna una delle costanti LIBERO, ELIMINATO o GIOCATO che
 * descrivono lo stato di un giocatore. In particolare:
 * -lo stato LIBERO corrisponde a un giocatore che ha la possibilit&agrave; di
 * giocare in questo turno o in turni successivi (non &egrave; mai stato eliminato
 * e non ha ancora giocato in questo turno);
 * -lo stato ELIMINATO indica che il giocatore &egrave; gi&agrave; stato battuto in un
 * turno precedente a questo;
 * -lo stato GIOCATO indica che il giocatore ha preso parte a una partita
 * nel turno corrente.
 *
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file delle
 * partite, completo di percorso.
 * @param id il numero identificativo del giocatore
 * @return LIBERO, ELIMINATO, GIOCATO
 */
int giocStatoGiocatore(const char* nomeFile, int id){
 	int partite = parContaPartite(nomeFile);
 	int questoTurno = parTurnoCorrente(nomeFile);
 	assert(questoTurno >0 && questoTurno <100);
 	//Questo è il risultato di questa funzione.
 	int stato = LIBERO;
 	
 	for (int i=0; i<partite; i++){
 		Partita* questa = parDettagliPartita(nomeFile, partite-i-1);
 		assert(questa!=NULL);

		if (questoTurno == questa->turno){
 			/*Stiamo leggendo una partita di questo turno. Dobbiamo solo
 			controllare se il giocatore ha giocato o meno */
 			if (id == questa->idG1 || id == questa->idG2){
 				stato = GIOCATO;
			    delete questa;
		 	    break;
 			}
 		}
 		
 		//Se arrivo qui, devo solo controllare che il giocatore non sia già
 		//stato eliminato. Controllo se ha perso la partita corrente.
 		if (((id == questa->idG1) && (questa->risG1 < questa->risG2)) ||
 		    ((id == questa->idG2) && (questa->risG1 > questa->risG2))){
 			stato = ELIMINATO;
	 	    delete questa;
			break;
 		}
 		delete questa;
 	}
    return stato;
}

/**
 * Ritorna una struttura che descrive il giocatore identificato dall'id
 * specificato.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * @param id l'id del giocatore
 * @return una struttura Giocatore con i dettagli
 */
Giocatore* giocDettagliGiocatore(const char* nomeFile, int id){
	assert(nomeFile!=NULL && (strcmp(nomeFile, "")!=0));
	assert(id>=0 && id<fileContaRecord(nomeFile));
	Giocatore* risultato = new Giocatore;
	risultato->id = id;
	risultato->nome = fileLeggiCampo(nomeFile, 0, id);
	return risultato;
}

/**
 * Ritorna una lista nella quale ogni elemento &egrave; una struttura giocatore
 * con i dettagli relativi a un giocatore libero.
 * 
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso.
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * 
 */
Lista* giocGiocatoriLiberi(const char* nomeFileGiocatori, const char* nomeFilePartite){
	assert(nomeFilePartite!=NULL && (strcmp(nomeFilePartite, "")!=0));
	assert(nomeFileGiocatori!=NULL && (strcmp(nomeFileGiocatori, "")!=0));
	
	int numeroGiocatori = fileContaRecord(nomeFileGiocatori);
	Lista* risultato = listeCreaLista(NULL);
	
	if(!fileEsiste(nomeFilePartite)){         
    	for (int i=0; i<numeroGiocatori; i++){
     		listeAggiungiElemento(risultato, 
			giocDettagliGiocatore(nomeFileGiocatori, i));
        }
      }
      else{
	     for (int i=0; i<numeroGiocatori; i++){
		     if (giocStatoGiocatore(nomeFilePartite, i) == LIBERO){
		     	Giocatore*g = giocDettagliGiocatore(nomeFileGiocatori, i);
		     	listeAggiungiElemento(risultato, g);
		    }
	    }
    }
	return risultato;
}

/**
 * Ritorna il nome del giocatore avente l'id specificato.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * @param idGiocatore id del giocatore
 * @return il nome del giocatore
 */
char* giocNomeGiocatore(const char* nomeFile, int idGiocatore){
      return fileLeggiCampo(nomeFile, 0, idGiocatore);
}
 
/**
 * Inserisce un nuovo giocatore nel Torneo.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * @param nomeGiocatore il nome del giocatore
 */
void giocNuovoGiocatore(const char* nomeFile, char* nomeGiocatore){
     fileAggiungiCampo(nomeFile, nomeGiocatore);
     fileNuovoRecord(nomeFile);
}

/**
 * Fornisce informazioni sul ruolo di un giocatore in una data partita.
 * Ritorna una delle costanti simboliche HAVINTO, HAPERSO o NONGIOCA
 * a seconda se il giocatore risulti vincitore, perdente o non abbia preso
 * parte a una partita.
 * 
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso.
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * @param idGiocatore il giocatore di cui si vuole conoscere il ruolo
 * @param idPartita la partita da esaminare
 */
int giocHaGiocato(const char* nomeFileGiocatori, const char* nomeFilePartite, int idGiocatore, int idPartita){
	assert(nomeFilePartite!=NULL && (strcmp(nomeFilePartite, "")!=0));
	assert(nomeFileGiocatori!=NULL && (strcmp(nomeFileGiocatori, "")!=0));
	assert(idGiocatore>=0 && idGiocatore<fileContaRecord(nomeFileGiocatori));
	assert(idPartita>=0 && idPartita<parContaPartite(nomeFilePartite));
	
	Partita* p = parDettagliPartita(nomeFilePartite, idPartita);
	int risultato = NONGIOCA;
	if (idGiocatore == p->idG1){
		if (p->risG1>p->risG2){
			risultato = HAVINTO;
		}
		else{
			risultato = HAPERSO;
		}
	}
	else{
		if (idGiocatore == p->idG2){
			if (p->risG2>p->risG1){
				risultato = HAVINTO;
			}
			else{
				risultato = HAPERSO;
			}
		}
	}
	delete p;
	return risultato;
}
