/* Tennis Tournament Management System 2005 by Silvio Moioli
    and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it).

	


	ATTENZIONE: questa è la versione DEBUG del programma, pertanto non
	contiene alcuni dei cambiamenti che sono inglogati nella versione
	RELEASE. Si consiglia di usare sempre e solo la versione RELEASE.

	Questa versione è stata pubblicata SOLO per documentare le fasi intermedie
	del lavoro con le assertions e i programmi di prova (come descritto nella
	documentazione)!!!


 */
#include <assert.h>
#include "liste.h"

/**
 * listaCrea: questa funzione ha il compito di creare una lista nuova;
 * Argomenti: primo  puntatore generico al primo elemento della lista
 *                     se questo vale NULL allora si crea una lista vuota;
 *   La funzione ritorna il puntatore alla lista creata.
 */

Lista* listeCreaLista(const void* primo)
{
    Lista* list = new Lista;
  
    if (primo==NULL) {
        list->testa=NULL;
        list->coda=NULL;
        list->n=0;
    }            
    else {
        list->testa=new ElemLista;
        list->coda=list->testa;
        list->n=1;
        list->testa->elemento=primo;
        list->testa->prossimo=NULL;
    }            
    return list;
} 

/**
*   listaDistruggi: ha il compito di deallocare un'intera lista.
*   Argomenti: l  Puntatore alla lista da deallocare;
*   E' una procedura e quindi non ritorna nulla.
*/
void listeDistruggiLista(Lista* l)
{
    assert(l!=NULL);
    if (l->n!=0){
	    ElemLista* attuale=l->testa;    
    	ElemLista* prossimo;
        while (attuale!=NULL) {
            prossimo=attuale->prossimo;
            delete attuale;
            attuale=prossimo;    
        }
    }
    delete l;
}

/**
*   listaAggiungi: ha il compito di inserire un elemento in coda alla lista
*   Argomenti: l  puntatore alla lista nella quale inserire l'elemento
*              elemento  puntatore void all'elemento da aggiungere 
*/
void listeAggiungiElemento(Lista* l, const void* elemento)
{
    assert(l!=NULL);
    assert(elemento!=NULL);
    if (l->n==0) {
        l->testa=new ElemLista;
        l->testa->elemento=elemento;
        l->testa->prossimo=NULL;
        l->coda=l->testa;
    }  
    else{             
	    ElemLista* attuale=l->testa;
	    ElemLista* precedente = NULL;
        while (attuale!=NULL) {    
                precedente=attuale;        
                attuale=attuale->prossimo;
        }
        ElemLista* nuovo = new ElemLista;
        precedente->prossimo=nuovo;
        nuovo->elemento=elemento;
        nuovo->prossimo=NULL;
        l->coda=nuovo;
    }        
    l->n++;
}    

/*
*   listaLeggi: ha il compito di leggere l'i-esimo elemento di una lista
*   Argomenti: l  puntatore alla lista dalla quale leggere l'elemento;
*              i  indice dell'elemento da leggere
*   Ritorna al main il puntatore generico all'elemento letto.
*/
const void* listeLeggiElemento(Lista* l, int i)
{
    assert(l!=NULL);
    assert(i>=0 && i<l->n);
    ElemLista* supp=l->testa;
    for (int j=0; j<i; j++){    
        assert(supp!=NULL);
        supp=supp->prossimo;
    }  
    assert(supp!=NULL);
    return supp->elemento;
}

/*
*   listaScrivi: ha il compito di sovrascrivere l'i-esimo elemento di
                 una lista  
*   Argomenti: l  puntatore alla lista nella quale sovrascrivere l'elemento;
*              i  indice dell'elemento da sovrascrivere
*              elemento  puntatore generico all'elemento da sovrascrivere; 
*   Attenzione: RICORDARSI DI DEALLOCARE L'ELEMENTO CHE
* 	VERRA' SOVRASCRITTO!                
*/
void listeScriviElemento(Lista* l, int i, const void* elemento)
{
    assert(l!=NULL);
    assert(i>=0 && i<l->n);
    ElemLista* supp=l->testa;
    for (int j=0; j<i; j++){    
        assert(supp!=NULL);
        supp=supp->prossimo;
    }        
    assert(supp!=NULL);
    supp->elemento = elemento;
}
