/* Tennis Tournament Management System 2005 by Silvio Moioli
    and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it).

	


	ATTENZIONE: questa è la versione DEBUG del programma, pertanto non
	contiene alcuni dei cambiamenti che sono inglogati nella versione
	RELEASE. Si consiglia di usare sempre e solo la versione RELEASE.

	Questa versione è stata pubblicata SOLO per documentare le fasi intermedie
	del lavoro con le assertions e i programmi di prova (come descritto nella
	documentazione)!!!


 */
 #include "file.h"
 #include "partita.h"
 
 /**
 * Ritorna il numero progressivo di questo turno.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file
 * @return il numero del turno corrente
 */
int parTurnoCorrente(const char* nomeFile){
	if (fileEsiste(nomeFile)){
		//Indice dell'ultimo record nel file
//		char* turno = fileLeggiCampo(nomeFile, 0, parContaPartite(nomeFile) -1);
		int records = fileContaRecord(nomeFile);
		char* turno = fileLeggiCampo(nomeFile, 0,  records-1);
    	assert(strcmp(turno,"")!=0);
        assert(strcmp(turno,"*")!=0);
	    int risultato = atoi(turno);
	  	delete[] turno;
    	
    	if (fileUltimoCarattere(nomeFile) == '*'){
    		risultato++;
    	}
	    assert(risultato >0 && risultato<100);
	    return risultato;
	}
	return 1;
}

/**
 * Ritorna il numero totale di partite giocate nel Torneo.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file
 * @return il numero di partite
 */
int parContaPartite(const char* nomeFile){
	if (fileEsiste(nomeFile)){
		//Indice dell'ultimo record nel file
		int fine=fileContaRecord(nomeFile);
		char* turno = fileLeggiCampo(nomeFile, 0, fine-1);
		/*Usiamo la stringa * per identificare un nuovo turno senza ancora
		nessuna partita*/ 
		bool turnoVuoto = (strcmp(turno,"*") == 0);
	   	delete[] turno;
	    if (turnoVuoto){
	    	//E' stata giocata una partita in meno dei record nel file, l'ultimo
	    	//e' un record vuoto per indicare la fine del turno
	    	return fine-1;
	    }
	    return fine;
	}
	return 0;
}

/**
 * Ritorna il numero di partite giocate in un turno.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @param turno il turno del quale si vogliono avere informazioni
 * @return il numero di partite giocate nel turno
 */
int parPartiteGiocate(const char* nomeFile, int turno){
	if (fileEsiste(nomeFile)){
		int record=fileContaRecord(nomeFile);
	    int partiteGiocate=0;
	    for (int i=0;i<record;i++){
	    	char*s = fileLeggiCampo(nomeFile, 0, record-i-1);
	    	if (strcmp(s,"*") != 0){
		    	int turnoAttuale = atoi(s);
		    	assert(turnoAttuale>0);
		        delete[] s;
		        
		        if (turnoAttuale==turno){
		        	partiteGiocate++;
		        }
		        if (turnoAttuale<turno){
		        	break;
		        }
	    	}
	    }
	    return partiteGiocate;
	}
	return 0;
}

/**
 * Carica i dati relativi a una partita da un file a una struttura in memoria.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da 
 * leggere
 * @param id numero identificativo della partita
 */
Partita* parDettagliPartita(const char* nomeFile, int id){
	/*TODO: ottimizzare creando una funzione nella file.h che permette
	di leggere un intero record alla volta*/

	assert(id>=0 && id<parContaPartite(nomeFile));

	Partita* letta = new Partita;
	
	char* tmp = fileLeggiCampo(nomeFile, 0, id);
	letta->turno = atoi(tmp);
	assert(letta->turno>0 && letta->turno<100);
	delete []  tmp;
	
	tmp = fileLeggiCampo(nomeFile, 1, id);
	letta->idG1 = atoi(tmp);
	assert(letta->idG1>=0 && letta->idG1<100);
	delete []  tmp;
	
	tmp = fileLeggiCampo(nomeFile, 2, id);
	letta->idG2 = atoi(tmp);
	assert(letta->idG2>=0 && letta->idG2<100);
	delete []  tmp;
	
	tmp = fileLeggiCampo(nomeFile, 3, id);
	letta->risG1 = atoi(tmp);
	assert(letta->risG1>=0 && letta->risG1<7);
	delete []  tmp;

	tmp = fileLeggiCampo(nomeFile, 4, id);
	letta->risG2 = atoi(tmp);
	assert(letta->risG2>=0 && letta->risG2<7);
	delete []  tmp;
	
	return letta;
}

/**
 * Carica i dati relativi a tutte le partite di un certo turno e li ritorna
 * in una lista opportuna.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da 
 * leggere
 * @param turno il numero del turno di cui caricare i dati delle partite
 */
Lista* parListaPartiteGiocate(const char* nomeFile, int turno){
	assert(turno<=parTurnoCorrente(nomeFile));
	Lista* risultato = listeCreaLista(NULL);
	int record=fileContaRecord(nomeFile);
    
    for (int i=0;i<record;i++){
    	char*s = fileLeggiCampo(nomeFile, 0, record-i-1);
    	if (strcmp(s,"*") != 0){   	
	    	int turnoAttuale = atoi(s);
	    	assert(turnoAttuale>0);
	        delete[] s;
	        
	        if (turnoAttuale==turno){
	        	Partita* nuovoElemento = parDettagliPartita(nomeFile, record-i-1);
	        	assert(nuovoElemento != NULL);
	        	listeAggiungiElemento(risultato, nuovoElemento);
	        }
		    if (turnoAttuale<turno){
		      	break;
		    }
    	}
    }
    return risultato;
}

/**
 * Aggiunge una nuova partita al file.
 * 
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file a 
 * cui aggiungere la partita
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file
 * dei giocatori
 * @param idG1 numero identificativo del primo concorrente
 * @param idG2 numero identificativo del secondo concorrente
 * @param risG1 punteggio del primo giocatore a fine partita
 * @param risG2 punteggio del secondo giocatore a fine partita
 */
void parNuovaPartita(const char* nomeFileGiocatori, const char* nomeFilePartite,
 int idG1, int idG2, int risG1, int risG2)
{
    assert(idG1>=0);
    assert(idG1<fileContaRecord(nomeFileGiocatori));
    assert(idG2>=0);
    assert(idG2<fileContaRecord(nomeFileGiocatori));
    assert(risG1>=0);
    assert(risG1<=6);
    assert(risG2>=0);
    assert(risG2<=6);
    assert(risG1!=risG2);

	
    char *temp = new char[16];
   	snprintf(temp, 16, "%d", parTurnoCorrente(nomeFilePartite));
	if (fileEsiste(nomeFilePartite) && fileUltimoCarattere(nomeFilePartite)=='*'){
//		    snprintf(temp, 16, "%d", parTurnoCorrente(nomeFilePartite)+1);
			fileSovrascriviUltimoCampo(nomeFilePartite, temp);
	}
	else{
			fileAggiungiCampo(nomeFilePartite, temp);
	}
   
    snprintf(temp, 16, "%d", idG1);
    fileAggiungiCampo(nomeFilePartite, temp);
    snprintf(temp, 16, "%d", idG2);
    fileAggiungiCampo(nomeFilePartite, temp);
    snprintf(temp, 16, "%d", risG1);
    fileAggiungiCampo(nomeFilePartite, temp);
    snprintf(temp, 16, "%d", risG2);
    fileAggiungiCampo(nomeFilePartite, temp);
    delete[] temp;
    fileNuovoRecord(nomeFilePartite);
}

/**
 * Chiude il turno corrente aprendone uno nuovo.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da 
 * leggere
 */
void parNuovoTurno(const char *nomeFile)
{
     fileAggiungiCampo(nomeFile, "*");
}
