/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
/**
 * Liberia file.h
 * Gestisce l'I/O su file CSV (Comma Separated Values).
 * N.B.: Questa &egrave; l'implementazione "Release", che non 
 * contiene molti controlli  extra (assertions) presenti nella
 * versione "Debug".
 */
#include <cstdlib>
#include <fstream>
#include <iostream>
using namespace std;

#ifndef TT_FILE
#define TT_FILE

/**
 * Aggiungere un campo in coda al file.
 * Per "campo" si intende una stringa memorizzata in un file CSV.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @param s stringa terminata da ' \\0' con il campo da inserire
 */
void fileAggiungiCampo(const char* nomeFile, const char* s);

/**
 * Aggiungere un campo in coda al file, sovrascrivendo l'ultimo
 * carattere dell'ultimo record presente.
 * Per "campo" si intende una stringa memorizzata in un file CSV.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @param s stringa terminata da ' \\0' con il campo da inserire
 */
void fileAggiungiCampoUltimoChar(const char* nomeFile, const char* s);

/**
 * Chiude il record corrente. La prossima operazione di fileAggiungiCampo
 * avverr&agrave; su un nuovo record.
 * Per "campo" si intende una stringa memorizzata in un file CSV.
 * Per "record" si intende una riga di un file CSV, composta da uno o pi&ugrave;
 * "campi".
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il record, completo di percorso.
 */
void fileNuovoRecord(const char* nomeFile);

/**
 * Legge e ritorna l'x-esimo campo nell'y-esimo record del file in formato
 * stringa.
 * Note: le coordinate x ed y funzionano come negli array multidimensionali
 * in C, sono numerate quindi da 0 a n-1, dove n &egrave; il numero
 * degli elementi.
 * Per "campo" si intende una stringa memorizzata in un file CSV.
 * Per "record" si intende una riga di un file CSV, composta da uno o pi&ugrave;
 * "campi".
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * cui leggere il campo, completo di percorso.
 * @param x l'indice del campo
 * @param y l'indice del record
 * @return una stringa appena allocata contenente il dato letto
 */
char* fileLeggiCampo(const char* nomeFile, int x, int y);

/**
 * Conta il numero di record all'interno del file.
 * Per "record" si intende una riga di un file CSV, composta da uno o pi&ugrave;
 * "campi".
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file,
 *  completo di percorso.
 * @return il numero di record presenti nel file
 */
int fileContaRecord(const char* nomeFile);

/**
 * Determina l'esistenza di un file.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * controllare, completo di percorso.
 * @return true se il file esiste, false altrimenti.
 */
bool fileEsiste(const char* nomeFile);

/**
 * Legge e ritorna l'ultimo carattere in un file, prima dell'eof.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file,
 *  completo di percorso.
 * @return l'ultimo carattere nel file
 */
char fileUltimoCarattere(const char* nomeFile);

#endif
