/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
#include "giocatore.h"
#include "partita.h"
#include "file.h"
#include "liste.h"

/* Per i commenti sulle funzioni pubbliche e i loro i parametri (che sono 
  stati eliminati per  evitare inutili duplicazioni e spreco di spazio)
  si rimanda all'header  della libreria. */

Giocatore* giocDettagliGiocatore(const char* nomeFile, int id){
            Giocatore* risultato = new Giocatore;
    risultato->id = id;
    risultato->nome = fileLeggiCampo(nomeFile, 0, id);
    return risultato;
}

int giocStatoGiocatore(const char* nomeFile, int id){
    int partite = parContaPartite(nomeFile);
    int questoTurno = parTurnoCorrente(nomeFile);
        //Questo e' il risultato di questa funzione.
    int stato = LIBERO;

    for (int i = 0; i < partite; i++){
        Partita* questa = parDettagliPartita(nomeFile, partite - i - 1);
        
        if (questoTurno == questa->turno){
            /* Stiamo leggendo una partita di questo turno. Dobbiamo solo
            controllare se il giocatore ha giocato o meno */
            if (id == questa->idG1 || id == questa->idG2){
                stato = GIOCATO;
                delete questa;
                break;
            }
        }

        //Se arrivo qui, devo solo controllare che il giocatore non sia gia'
        //stato eliminato. Controllo se ha perso la partita corrente.
        if (((id == questa->idG1) && (questa->risG1 < questa->risG2)) ||
                ((id == questa->idG2) && (questa->risG1 > questa->risG2))){
            stato = ELIMINATO;
            delete questa;
            break;
        }
        delete questa;
    }
    return stato;
}

Lista* giocGiocatoriLiberi(const char* nomeFileGiocatori,
    const char* nomeFilePartite){
    int numeroGiocatori = fileContaRecord(nomeFileGiocatori);
    Lista* risultato = listeCreaLista(NULL);

    if (!fileEsiste(nomeFilePartite)){
        for (int i = 0; i < numeroGiocatori; i++){
            listeAggiungiElemento(risultato,
                                  giocDettagliGiocatore(nomeFileGiocatori, i));
        }
    }
    else {
        for (int i = 0; i < numeroGiocatori; i++){
            if (giocStatoGiocatore(nomeFilePartite, i) == LIBERO){
                Giocatore*g = giocDettagliGiocatore(nomeFileGiocatori, i);
                listeAggiungiElemento(risultato, g);
            }
        }
    }
    return risultato;
}

char* giocNomeGiocatore(const char* nomeFile, int idGiocatore){
    return fileLeggiCampo(nomeFile, 0, idGiocatore);
}

void giocNuovoGiocatore(const char* nomeFile, char* nomeGiocatore){
    fileAggiungiCampo(nomeFile, nomeGiocatore);
    fileNuovoRecord(nomeFile);
}

int giocHaGiocato(const char* nomeFileGiocatori,
              const char* nomeFilePartite,
    int idGiocatore, int idPartita){
    Partita* p = parDettagliPartita(nomeFilePartite, idPartita);
    int risultato = NONGIOCA;
    if (idGiocatore == p->idG1){
        if (p->risG1 > p->risG2){
            risultato = HAVINTO;
        }
        else {
            risultato = HAPERSO;
        }
    }
    else {
        if (idGiocatore == p->idG2){
            if (p->risG2 > p->risG1){
                risultato = HAVINTO;
            }
            else {
                risultato = HAPERSO;
            }
        }
    }
    delete p;
    return risultato;
}
