/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
/**
 * Liberia giocatore.h
 * Gestisce le operazioni ad alto livello sui giocatori.
 * N.B.: Questa &egrave; l'implementazione "Release", che non 
 * contiene molti controlli  extra (assertions) presenti nella
 * versione "Debug".
 */
#include <cstdlib>
using namespace std;

#ifndef TT_LISTE
#define TT_LISTE

/** Descrive un elemento della lista. */
struct ElemLista {
    /** Puntatore al contenuto dell'elemento */
    const void *elemento;
    /** Puntatore al prossimo elemento nella lista o NULL */
    ElemLista *prossimo;
};

/** Descrive un'intera lista. */
struct Lista {
    /** Puntatore al primo elemento della lista */
    ElemLista *testa;
    /** Puntatore all'ultimo elemento della lista */
    ElemLista *coda;
    /** Numero di elementi nella lista (sola lettura) */
    int n;
};

/** 
 * Crea una nuova lista, e opzionalmente aggiunge il primo elemento.
 * 
 * @param primo il primo elemento oppure NULL per una lista vuota
 * @return la nuova Lista appena creata
 */
Lista *listeCreaLista(const void *primo);

/**
 * Dealloca una lista e i rispettivi elementi.
 * N.B.: <b>NON</b> dealloca i contenuti della lista!
 * 
 * @param l la lista da deallocare
 */
void listeDistruggiLista(Lista * l);

/**
 * Accoda un nuovo elemento alla lista.
 * 
 * @param l la lista a cui aggiungere l'elemento
 * @param elemento l'elemento da aggiungere
 */
void listeAggiungiElemento(Lista * l, const void *elemento);

/**
 * Legge l'i-esimo elemento e ne ritorna il puntatore. Gli indici
 * della lista sono trattati come negli array monodimensionali del C:
 * il primo elemento ha indice 0, l'ultimo n-1 (con n elementi).
 * 
 * @param l la lista da cui leggere
 * @param i l'indice dell'elemento da leggere
 * @return il puntatore all'elemento letto
 */
const void *listeLeggiElemento(Lista * l, int i);

/**
 * Sovrascrive l'i-esimo elemento della lista. Gli indici
 * della lista sono trattati come negli array monodimensionali del C:
 * il primo elemento ha indice 0, l'ultimo n-1 (con n elementi).
 * N.B.: <b>NON </b> dealloca l'elemento sovrascritto!
 * 
 * @param l la lista a cui cambiare l'elemento
 * @param i indice dell'elemento da sovrascrivere
 * @param elemento il nuovo elemento
 */
void listeScriviElemento(Lista * l, int i, const void *elemento);

#endif
