/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
#include "menu.h"

/**
 * Permette di cambiare un giocatore in una partita (se possibile) con un altro
 * libero. Chiede una scelta all'utente in merito, se necessario.
 * 
 * @param nomeFileGiocatori stringa terminata da '\\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da '\\0' con il nome del file 
 * delle partite, completo di percorso
 * @param p dettagli della partita a cui cambiare il giocatore
 * @param daCambiare 1 o 2, a seconda se il giocatore da cambiare &egrave; il
 * primo o il secondo concorrente.
 * @return l'id del nuovo giocatore
 */
int menuCambiaGiocatore(const char* nomeFileGiocatori, 
                            const char* nomeFilePartite, Partita* p, int daCambiare){
    int risultato = 0;
    //Lista dei giocatori liberi (tutti)
    Lista* liberi = giocGiocatoriLiberi(nomeFileGiocatori, nomeFilePartite);
    if (liberi->n < 3){
        //Non e' possibile effettuare altra scelta se non i giocatori correnti.
        if (daCambiare == 1){
            risultato = p->idG1;
        }
        else {
            risultato = p->idG2;
        }
    }
    else {
        cout<<"Scegli il concorrente"<<endl;
        uiStampaLinea();
        //Lista delle voci di menu
        Lista* menu = listeCreaLista(NULL);
        //Lista degli id dei giocatori, tranne quelli gia' scelti
        Lista* possibili = listeCreaLista(NULL);
        for (int i = 0;i < liberi->n;i++){
            Giocatore* g = (Giocatore *)listeLeggiElemento(liberi, i);
            if (g->id != p->idG1 && g->id != p->idG2){
                listeAggiungiElemento(menu, g->nome);
                listeAggiungiElemento(possibili, &(g->id));
            }
        }
        uiStampaMenu(menu);

        if (liberi->n == 3){
            risultato = *( (int*)listeLeggiElemento(possibili, 0));
        }
        else {
            int scelta = uiChiediScelta(menu->n);
            risultato = *( (int*)listeLeggiElemento(possibili, scelta - 1) );
        }
        listeDistruggiLista(menu);
        listeDistruggiLista(possibili);
    }
    //Ripulisce e ritorna
    for (int i = 0; i < liberi->n; i++){
        Giocatore* g = (Giocatore*) listeLeggiElemento(liberi, i);
        delete[] g->nome;
        delete g;
    }
    listeDistruggiLista(liberi);
    return risultato;
}

/**
 * Permette di cambiare un risultato in una partita.
 * Chiede una scelta all'utente in merito.
 * 
 * @param p dettagli della partita a cui cambiare il risultato
 * @param daCambiare 1 o 2, a seconda se il giocatore cui cambiare il risultato
 * da cambiare &egrave; il primo o il secondo concorrente.
 * @return il nuovo risultato
 */
int menuCambiaRisultato(Partita* p, int daCambiare){
    cout<<"Inserire il risultato della partita"<<endl;
    uiStampaLinea();
    int risultato = uiChiediScelta(6);
    if (daCambiare == 1 && risultato == p->risG2 ||
		    daCambiare == 2 && risultato == p->risG1){
        cout << "Nei tornei a eliminazione diretta non sono ammessi i pareggi." 
            << endl;
        uiChiediConferma();
	    if (daCambiare == 1){
            return p->risG1;
 		}
 		else{
            return p->risG2;
 		}
    }
    //La scelta dell'utente e' corretta
    return risultato;
}

/**
 * Visualizza un'ipotesi di nuova partita e permette all'utente di cambiare
 * concorrenti e risultati (ove possibile).
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso
 * @param p dettagli della partita di default
 * @return true se la proposta della partita &egrave; stata accettata dall'utente,
 * false altrimenti
 */
bool menuNuovaPartitaMostra(const char* nomeFileGiocatori,
                            const char* nomeFilePartite, Partita* p){
    bool uscita = false;
    bool risultato = false;
    while (!uscita){
        int turno = parTurnoCorrente(nomeFilePartite);
        int partite = parContaPartiteTurno(nomeFilePartite, turno);
        cout<<"Nuova partita nel turno numero: " << turno
        	<< " Giocate finora " << partite << " partite"<<endl;
        uiStampaLinea();
        char* tmp = giocNomeGiocatore(nomeFileGiocatori, p->idG1);
        cout << "Giocatore 1: " << tmp << " -- Risultato " << p->risG1 << endl;
        delete[] tmp;
        tmp = giocNomeGiocatore(nomeFileGiocatori, p->idG2);
        cout << "Giocatore 2: " << tmp << " -- Risultato " << p->risG2 << endl;
        delete[] tmp;

        Lista *menu = listeCreaLista("Cambia Giocatore 1");
        listeAggiungiElemento(menu, "Cambia Giocatore 2");
        listeAggiungiElemento(menu, "Imposta Risultato Giocatore 1");
        listeAggiungiElemento(menu, "Imposta Risultato Giocatore 2");
        listeAggiungiElemento(menu, "Conferma e Salva Parita");
        listeAggiungiElemento(menu, "Annulla Partita e torna al Menu Turno");
        uiStampaMenu(menu);
        listeDistruggiLista(menu);

        switch (uiChiediScelta(6)){
            case 1:
                p->idG1 = menuCambiaGiocatore(nomeFileGiocatori, nomeFilePartite,
                    p, 1);
                break;
            case 2:
                p->idG2 = menuCambiaGiocatore(nomeFileGiocatori, nomeFilePartite,
                    p, 2);
                break;
            case 3:
                p->risG1 = menuCambiaRisultato(p, 1);
                break;
            case 4:
                p->risG2 = menuCambiaRisultato(p, 2);
                break;
            case 5:
                uscita = true;
                risultato = true;
                break;
            case 6:
                uscita = true;
                risultato = false;
                break;
            }
    }
    return risultato;
}

/**
 * Controlla se sussistono le condizioni per iniziare una nuova partita.
 * In caso positivo, crea una partita ipotetica e la propone all'utente per
 * i suoi cambiamenti, altrimenti avvisa dell'impossibilit&agrave;
 * di creare una nuova aprtita.
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuNuovaPartita(const char* nomeFileGiocatori,
                            const char* nomeFilePartite){
    if ((!fileEsiste(nomeFileGiocatori)) || (fileContaRecord(nomeFileGiocatori) < 2)){
        cout << "Non ci sono abbastanza giocatori iscritti al torneo!" << endl;
        uiChiediConferma();
    }
    else {
        Lista *liberi = giocGiocatoriLiberi(nomeFileGiocatori, nomeFilePartite);
        if (liberi->n < 2){
            for (int i = 0; i < liberi->n; i++){
                const Giocatore* g = (Giocatore* ) listeLeggiElemento(liberi, i);
                delete[] g->nome;
                delete g;
            }
            listeDistruggiLista(liberi);
            cout << "Non ci sono abbastanza giocatori liberi!" << endl;
            uiChiediConferma();
        }
        else {
            //Ci sono abbastanza giocatori e sono liberi. Creiamo la partita di default.
            Giocatore *gioc1 = (Giocatore *) listeLeggiElemento(liberi, 0);
            Giocatore *gioc2 = (Giocatore *) listeLeggiElemento(liberi, 1);
            Partita *nuova = new Partita;
            nuova->turno = parTurnoCorrente(nomeFilePartite);
            nuova->idG1 = gioc1->id;
            nuova->idG2 = gioc2->id;
            nuova->risG1 = 1;
            nuova->risG2 = 2;
            for (int i = 0; i < liberi->n; i++){
                Giocatore* g = (Giocatore* ) listeLeggiElemento(liberi, i);
                delete[] g->nome;
                delete g;
            }
            listeDistruggiLista(liberi);
            if (menuNuovaPartitaMostra(nomeFileGiocatori, nomeFilePartite, nuova)){
                //Salviamo le scelte su file
                parNuovaPartita(nomeFileGiocatori, nomeFilePartite, nuova->idG1,
                    nuova->idG2, nuova->risG1, nuova->risG2);
            }
            delete nuova;
        }
    }
}

/**
 * Stampa la lista delle partite giocate in un turno, con i rispettivi risultati.
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso
 * @param turno il numero del turno di cui stampare le partite
 */
 void menuPartiteTurno(const char* nomeFileGiocatori,
                           const char* nomeFilePartite, int turno){
        int partite = parContaPartiteTurno(nomeFilePartite, turno);
    if (partite > 0){
        cout << "Nel turno numero " << turno << " sono state giocate ";
        cout << partite << " partite"<<endl;
        uiStampaLinea();
        
        Lista* giocate = parListaPartiteGiocate(nomeFilePartite, turno);
        for (int i = 0;i < giocate->n;i++){
            Partita* partita = (Partita*)listeLeggiElemento(giocate, i);
            char* temp = giocNomeGiocatore(nomeFileGiocatori, partita->idG1);
            cout << temp;
            delete[] temp;
            cout << " " << partita->risG1 << endl;
            temp = giocNomeGiocatore(nomeFileGiocatori, partita->idG2);
            cout << temp;
            delete[] temp;
            cout << " " << partita->risG2 << endl << endl;
            delete partita;
        }
        listeDistruggiLista(giocate);
    }
    else {
        cout << "Non sono state giocate partite nel turno " << turno << endl;
    }
    uiChiediConferma();
}

/**
 * Chiude (se possibile) il turno corrente.
 * 
 * @param nomeFileGiocatori stringa terminata da '\\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da '\\0' con il nome del file 
 * delle partite, completo di percorso
 * @param turno il numero del turno corrente
 */
void menuChiudiTurno(const char* nomeFileGiocatori,
                           const char* nomeFilePartite, int turno){
    if (parContaPartiteTurno(nomeFilePartite, turno) != 0){
        parNuovoTurno(nomeFilePartite);
        cout << "Turno Chiuso." << endl;
    }
    else {
        cout << "Per chiudere il Turno bisogna aver giocato almeno una partita!"
            << endl;
    }
    uiChiediConferma();
}

/**
 * Chiede all'utente di specificare un Turno (se necessario) di cui verranno
 * stampate le partite.
 * 
 * @param nomeFileGiocatori stringa terminata da '\\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da '\\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuTurniPassati(const char* nomeFileGiocatori,
                           const char* nomeFilePartite){
    int turno = parTurnoCorrente(nomeFilePartite);
    if (turno > 2){
        cout<<"Visualizzazione turni giocati finora"<<endl;
        uiStampaLinea();
        for (int i = 1;i < turno;i++){
            cout << "Turno " << i << ": giocate " << parContaPartiteTurno(nomeFilePartite, i) << " partite" << endl;
        }
        cout << "Immetti il numero del turno da visualizzare:" << endl;
        menuPartiteTurno(nomeFileGiocatori, nomeFilePartite,
            uiChiediScelta(turno - 1));
    }
    else {
        if (turno == 2){
            menuPartiteTurno(nomeFileGiocatori, nomeFilePartite, 1);
        }
        else {
            cout << "Siamo al primo turno, non ci sono turni precedenti." << endl;
            uiChiediConferma();
        }
    }
}

/**
 * Chiede all'utente il nome di un giocatore da aggiungere e lo aggiunge al file.
 * 
 * @param nomeFileGiocatori stringa terminata da '\\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da '\\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuAggiungiGiocatore(const char* nomeFileGiocatori,
                           const char* nomeFilePartite){
    cout<<"Inserisci il nome del Giocatore"<<endl;
    uiStampaLinea();
    char *nomeGiocatore = uiChiediStringa();
    giocNuovoGiocatore(nomeFileGiocatori, nomeGiocatore);
    delete[] nomeGiocatore;
    cout << "Giocatore inserito" << endl;
}

/**
 * Stampa l'elenco dei giocatori nel Torneo.
 * 
 * @param nomeFileGiocatori stringa terminata da '\\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da '\\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuGiocatoriTorneo(const char* nomeFileGiocatori,
                           const char* nomeFilePartite){
    if (fileEsiste(nomeFileGiocatori) && fileContaRecord(nomeFileGiocatori) > 0){
        int numGiocatori = fileContaRecord(nomeFileGiocatori);
        cout << "A questo torneo partecipano " << numGiocatori << " giocatori."<<endl;
        uiStampaLinea();
        cout << endl;
        for (int i = 0;i < numGiocatori;i++){
            char* nome = giocNomeGiocatore(nomeFileGiocatori, i);
            cout << nome << endl;
            delete[] nome;
        }
    }
    else {
        cout << "A questo Torneo non partecipa alcun giocatore." << endl;
    }
    uiChiediConferma();
}

/**
 * Stampa l'elenco degli avversari di un giocatore nel Torneo.
 * 
 * @param nomeFileGiocatori stringa terminata da '\\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da '\\0' con il nome del file 
 * delle partite, completo di percorso
 * @param giocatore l'id del giocatore da analizzare
 */
void menuVediCarrieraGiocatore(const char* nomeFileGiocatori,
                           const char* nomeFilePartite, int giocatore){
    char* nome = giocNomeGiocatore(nomeFileGiocatori, giocatore);
    cout << "Carriera del giocatore: " << nome << endl;
    delete[] nome;
    uiStampaLinea();
    cout << "Ha giocato contro:" << endl;
    bool haPerso = false;
    int partite = parContaPartite(nomeFilePartite);
    for (int i = 0;i < partite && !haPerso;i++){
        switch (giocHaGiocato(nomeFileGiocatori, nomeFilePartite, giocatore, i)){
            case NONGIOCA:
                break;
            case HAPERSO:
                cout << "E' stato battuto (ed eliminato) da: ";
                haPerso = true;
            case HAVINTO:
                Partita* part = parDettagliPartita(nomeFilePartite, i);
                char* nome = NULL;
                if (part->idG1 == giocatore){
                    nome = giocNomeGiocatore(nomeFileGiocatori, part->idG2);
                }
                else {
                    nome = giocNomeGiocatore(nomeFileGiocatori, part->idG1);
                }
                cout << nome << endl;
                delete[] nome;
                delete part;
                break;
        }
    }
    uiChiediConferma();
}

/**
 * Chiede all'utente di specificare un giocatore di cui mostrare la carriera
 * (elenco degli avversari contro cui ha giocato e risultati).
 * 
 * @param nomeFileGiocatori stringa terminata da '\\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da '\\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuCarrieraGiocatore(const char* nomeFileGiocatori,
                           const char* nomeFilePartite){
    if (fileEsiste(nomeFileGiocatori) && fileContaRecord(nomeFileGiocatori) > 0){
        if (fileEsiste(nomeFilePartite)){
            cout<<"Elenco dei giocatori nel Torneo"<<endl;
            uiStampaLinea();
            int numGiocatori = fileContaRecord(nomeFileGiocatori);
            for (int i = 0;i < numGiocatori;i++){
                char* nome = giocNomeGiocatore(nomeFileGiocatori, i);
                cout << i + 1 << "-" << nome << endl;
                delete[] nome;
            }
            if (numGiocatori == 1){
                menuVediCarrieraGiocatore(nomeFileGiocatori, nomeFilePartite, 1);
            }
            else {
                menuVediCarrieraGiocatore(nomeFileGiocatori, nomeFilePartite,
                    uiChiediScelta(numGiocatori) - 1);
            }
        }
        else {
            cout << "In questo Torneo non sono ancora state giocate partite."
                << endl;
            uiChiediConferma();
        }
    }
    else {
        cout << "A questo Torneo non partecipa alcun giocatore." << endl;
        uiChiediConferma();
    }
}

/* Per i commenti sulle funzioni pubbliche e i loro i parametri (che sono 
  stati eliminati per  evitare inutili duplicazioni e spreco di spazio)
  si rimanda all'header  della libreria. */
  
void menuPartite(const char* nomeFileGiocatori,
                           const char* nomeFilePartite){
    bool uscita = false;
    while (!uscita){
        int turno = parTurnoCorrente(nomeFilePartite);
        cout << "MENU' PARTITE: Turno: " << turno;
        cout << "; Giocate finora " <<
            parContaPartiteTurno(nomeFilePartite, turno) << " partite"<<endl;
        uiStampaLinea();

        Lista *menu = listeCreaLista("Nuova partita");
        listeAggiungiElemento(menu,
            "Visualizza le partite giocate in questo turno (e i risultati)");
        listeAggiungiElemento(menu, "Chiudi il turno");
        listeAggiungiElemento(menu,
            "Visualizza i dettagli di un turno passato");
        listeAggiungiElemento(menu, "Torna al Menu Principale");
        uiStampaMenu(menu);
        listeDistruggiLista(menu);

        switch (uiChiediScelta(5)){
            case 1:
                menuNuovaPartita(nomeFileGiocatori, nomeFilePartite);
                break;
            case 2:
                menuPartiteTurno(nomeFileGiocatori, nomeFilePartite, turno);
                break;
            case 3:
                menuChiudiTurno(nomeFileGiocatori, nomeFilePartite, turno);
                break;
            case 4:
                menuTurniPassati(nomeFileGiocatori, nomeFilePartite);
                break;
            case 5:
                uscita = true;
                break;
        }
    }
}

void menuGiocatori(const char* nomeFileGiocatori,
                           const char* nomeFilePartite){
    bool uscita = false;
    while (!uscita){
        int turno = parTurnoCorrente(nomeFilePartite);
        int partite = parContaPartite(nomeFilePartite);
        cout << "MENU' GIOCATORI: " << " Turno: " << turno <<
            "; Giocate finora " << partite << " partite"<<endl;
        uiStampaLinea();
        
        Lista *menu =
           listeCreaLista("Visualizza l'elenco dei giocatori in torneo");
        listeAggiungiElemento(menu,
            "Visualizza le partite e i risultati di un singolo giocatore");
        listeAggiungiElemento(menu, "Nuovo Giocatore");
        listeAggiungiElemento(menu, "Torna al Menu Principale");
        uiStampaMenu(menu);
        listeDistruggiLista(menu);
        switch (uiChiediScelta(4)){
            case 1:
                menuGiocatoriTorneo(nomeFileGiocatori, nomeFilePartite);
                break;
            case 2:
                menuCarrieraGiocatore(nomeFileGiocatori, nomeFilePartite);
                break;
            case 3:
                menuAggiungiGiocatore(nomeFileGiocatori, nomeFilePartite);
                break;
            case 4:
                uscita = true;
                break;
        }
    }
}

void menuRiepilogoTorneo(const char* nomeFileGiocatori,
                           const char* nomeFilePartite){
    int turno = parTurnoCorrente(nomeFilePartite);
    int partite = parContaPartite(nomeFilePartite);
    if (partite > 0){
        cout<<"Partite del torneo giocate finora:"<<endl;
        uiStampaLinea();
        for (int i = 1;i <= turno;i++){
            menuPartiteTurno(nomeFileGiocatori, nomeFilePartite, i);
        }
    }
    else {
        cout << "Non sono state giocate partite." << endl;
    }
    cout << "Riepilogo Torneo concluso." << endl;
    uiChiediConferma();
}
