/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
/**
 * Liberia menu.h
 * Include l'implementazione dell'interfaccia utente del programma
 * e la relativa logica "a menu".
 * N.B.: Questa &egrave; l'implementazione "Release", che non 
 * contiene molti controlli  extra (assertions) presenti nella
 * versione "Debug".
 */
#include "liste.h"
#include "file.h"
#include "giocatore.h"
#include "partita.h"
#include "ui.h"
using namespace std;

#ifndef TT_MENU
#define TT_MENU

/**
 * Visualizza un menu con le seguenti possibili scelte:
 * <ul> 
 * <li>Nuova partita</li>
 * <li>Visualizza le partite giocate in questo turno (e i risultati)</li>
 * <li>Chiudi il turno</li>
 * <li>Visualizza i dettagli di un turno passato</li>
 * <li>Torna al Menu Principale</li>
 *</ul>
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuPartite(const char* nomeFileGiocatori,
                              const char* nomeFilePartite);

/**
 * Visualizza un menu con le seguenti possibili scelte:
 * <ul> 
 * <li>Visualizza l'elenco dei giocatori in torneo</li>
 * <li>Visualizza le partite e i risultati di un singolo giocatore</li>
 * <li>Nuovo Giocatore</li>
 * <li>Torna al Menu Principale</li>
 *</ul>
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuGiocatori(const char* nomeFileGiocatori,
                              const char* nomeFilePartite);

/**
 * Visualizza un riepilogo dell'intero Torneo, con le partite giocate
 *  ad ogni turno.
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso
 */
void menuRiepilogoTorneo(const char* nomeFileGiocatori,
                              const char* nomeFilePartite);

#endif
