/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
#include "file.h"
#include "partita.h"

/* Per i commenti sulle funzioni pubbliche e i loro i parametri (che sono 
  stati eliminati per  evitare inutili duplicazioni e spreco di spazio)
  si rimanda all'header  della libreria. */

int parTurnoCorrente(const char* nomeFile){
    if (fileEsiste(nomeFile)){
        //Indice dell'ultimo record nel file
        int records = fileContaRecord(nomeFile);
        char* turno = fileLeggiCampo(nomeFile, 0, records - 1);
                        int risultato = atoi(turno);
        delete[] turno;

        if (fileUltimoCarattere(nomeFile) == '*'){
            risultato++;
        }
        return risultato;
    }
    return 1;
}

int parContaPartite(const char* nomeFile){
    if (fileEsiste(nomeFile)){
        //Indice dell'ultimo record nel file
        int fine = fileContaRecord(nomeFile);
        char* turno = fileLeggiCampo(nomeFile, 0, fine - 1);
        /* Usiamo la stringa * per identificare un nuovo turno senza ancora
        nessuna partita*/
        bool turnoVuoto = (strcmp(turno, "*") == 0);
        delete[] turno;
        if (turnoVuoto){
            /* E' stata giocata una partita in meno dei record nel file, 
                l'ultimo e' un record vuoto per indicare la fine del turno */
            return fine -1;
        }
        return fine;
    }
    return 0;
}

int parContaPartiteTurno(const char* nomeFile, int turno){
    if (fileEsiste(nomeFile)){
        int record = fileContaRecord(nomeFile);
        int partiteGiocate = 0;
        for (int i = 0;i < record;i++){
            char*s = fileLeggiCampo(nomeFile, 0, record - i - 1);
            if (strcmp(s, "*") != 0){
                int turnoAttuale = atoi(s);
                delete[] s;

                if (turnoAttuale == turno){
                    partiteGiocate++;
                }
                if (turnoAttuale < turno){
                    break;
                }
            }
        }
        return partiteGiocate;
    }
    return 0;
}

Partita* parDettagliPartita(const char* nomeFile, int id){
    /*TODO: ottimizzare creando una funzione nella file.h che permette
    di leggere un intero record alla volta */
    
    Partita* letta = new Partita;
    char* tmp = fileLeggiCampo(nomeFile, 0, id);
    letta->turno = atoi(tmp);
    delete [] tmp;
    tmp = fileLeggiCampo(nomeFile, 1, id);
    letta->idG1 = atoi(tmp);
    delete [] tmp;
    tmp = fileLeggiCampo(nomeFile, 2, id);
    letta->idG2 = atoi(tmp);
    delete [] tmp;
    tmp = fileLeggiCampo(nomeFile, 3, id);
    letta->risG1 = atoi(tmp);
    delete [] tmp;
    tmp = fileLeggiCampo(nomeFile, 4, id);
    letta->risG2 = atoi(tmp);
    delete [] tmp;

    return letta;
}

Lista* parListaPartiteGiocate(const char* nomeFile, int turno){
        Lista* risultato = listeCreaLista(NULL);
    int record = fileContaRecord(nomeFile);

    for (int i = 0;i < record;i++){
        char*s = fileLeggiCampo(nomeFile, 0, record - i - 1);
        if (strcmp(s, "*") != 0){
            int turnoAttuale = atoi(s);
            delete[] s;

            if (turnoAttuale == turno){
                Partita* nuovoElemento = 
                    parDettagliPartita(nomeFile, record - i - 1);
                 listeAggiungiElemento(risultato, nuovoElemento);
            }
            if (turnoAttuale < turno){
                break;
            }
        }
    }
    return risultato;
}

void parNuovaPartita(const char* nomeFileGiocatori,
                     const char* nomeFilePartite, int idG1, int idG2,
                     int risG1, int risG2){
    char *temp = new char[16];
    snprintf(temp, 16, "%d", parTurnoCorrente(nomeFilePartite));
    if (fileEsiste(nomeFilePartite) && 
        fileUltimoCarattere(nomeFilePartite) == '*'){
        fileAggiungiCampoUltimoChar(nomeFilePartite, temp);
    }
    else {
        fileAggiungiCampo(nomeFilePartite, temp);
    }

    snprintf(temp, 16, "%d", idG1);
    fileAggiungiCampo(nomeFilePartite, temp);
    snprintf(temp, 16, "%d", idG2);
    fileAggiungiCampo(nomeFilePartite, temp);
    snprintf(temp, 16, "%d", risG1);
    fileAggiungiCampo(nomeFilePartite, temp);
    snprintf(temp, 16, "%d", risG2);
    fileAggiungiCampo(nomeFilePartite, temp);
    delete[] temp;
    fileNuovoRecord(nomeFilePartite);
}

void parNuovoTurno(const char *nomeFile){
    fileAggiungiCampo(nomeFile, "*");
}
