/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
/**
 * Liberia partita.h
 * Operazioni ad alto livello su partite e turni del Torneo.
 * N.B.: Questa &egrave; l'implementazione "Release", che non 
 * contiene molti controlli  extra (assertions) presenti nella
 * versione "Debug".
 */
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <iostream>
#include "liste.h"
using namespace std;

#ifndef TT_PARTITA
#define TT_PARTITA

/**
 * Struttura che descrive una partita.
 */
struct Partita {
    /** Il numero del turno in cui questa partita &egrave; giocata*/
    int turno;
    /** idG1 numero identificativo del primo concorrente*/
    int idG1;
    /** iidG2 numero identificativo del secondo concorrente */
    int idG2;
    /** risG1 punteggio del primo giocatore a fine partita */
    int risG1;
    /** risG2 punteggio del secondo giocatore a fine partita */
    int risG2;
};

/**
 * Ritorna il numero progressivo di questo turno.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @return il numero del turno corrente
 */
int parTurnoCorrente(const char* nomeFile);

/**
 * Ritorna il numero totale di partite giocate nel Torneo.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file
 * @return il numero di partite
 */
int parContaPartite(const char* nomeFile);

/**
 * Ritorna il numero di partite giocate in questo turno.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file a
 * cui aggiungere il campo, completo di percorso.
 * @param turno il turno del quale si vogliono avere informazioni
 * @return il numero di partite giocate nel turno
 */
int parContaPartiteTurno(const char* nomeFile, int turno);

/**
 * Carica i dati relativi a una partita da un file a una struttura in memoria.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da 
 * leggere
 * @param id numero identificativo della partita
 * @return una struttura con i dettagli della partita richiesta
 */
Partita* parDettagliPartita(const char* nomeFile, int id);

/**
 * Carica i dati relativi a tutte le partite di un certo turno e li ritorna
 * in una lista opportuna.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da 
 * leggere
 * @param turno il numero del turno di cui caricare i dati delle partite
 * @return una Lista in cui ogni elemento &egrave; una struttura
 * Partita contenente i dettagli di una delle partite giocate in questo turno
 */
Lista* parListaPartiteGiocate(const char* nomeFile, int turno);

/**
 * Aggiunge una nuova partita al file.
 * 
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file a 
 * cui aggiungere la partita
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file
 * dei giocatori
 * @param idG1 numero identificativo del primo concorrente
 * @param idG2 numero identificativo del secondo concorrente
 * @param risG1 punteggio del primo giocatore a fine partita
 * @param risG2 punteggio del secondo giocatore a fine partita
 */
void parNuovaPartita(const char* nomeFileGiocatori,
                     const char* nomeFilePartite, int idG1, int idG2,
                     int risG1, int risG2);

/**
 * Chiude il turno corrente aprendone uno nuovo.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da 
 * leggere
 */
void parNuovoTurno(const char *nomeFile);

#endif
