/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
/**
 * Liberia ui.h
 * Funzioni utili nel disegno dell'interfaccia utente.
 * N.B.: Questa &egrave; l'implementazione "Release", che non 
 * contiene molti controlli  extra (assertions) presenti nella
 * versione "Debug".
 */
#include <stdlib.h>
#include "liste.h"

#ifndef TT_UI
#define TT_UI

/**
 * La massima lunghezza di una stringa leggibile da console.
 * Dovrebbe essere platform-dependant, &egrave; settata a un valore che
 * dovrebbe funzionare su tutte le piattaforme.
 */
#define MAXCONSOLEINPUT 128

/** 
 * Stampa una linea di 20 caratteri a video.
 */
void uiStampaLinea();

/**
 * Stampa un elenco di voci numerate per un menu.
 * 
 * @param voci una Lista contenente stringhe terminate da
 * '\\0' che rappresentano le voci di menu
 */
void uiStampaMenu(Lista* voci);

/**
 * Chiede all'utente di immettere un numero compreso tra
 * 1 e sceltaMax. 
 * 
 * @param sceltaMax massimo numero accettabile
 * @return la scelta dell'utente
 */
int uiChiediScelta(int sceltaMax);

/**
 * Chiede all'utente di digitare una stringa (lunghezza massima 
 * MAXCONSOLEINPUT compreso il '\\0') e la ritorna.
 * Non ammette stringhe vuote.
 * Usa esclusivamente funzioni "sicure" per evitare i buffer overflow.
 * 
 * @return la stringa immessa dall'utente
 */
char* uiChiediStringa();

/**
 * Chiede all'utente di battere un tasto e quindi Invio.
 * Questo purtroppo &egrave; l'unico modo concesso per chiedere
 * una conferma dalle librerie standard del C++.
 * Questa funzione potrebbe essere meglio reimplementata in
 * un contesto non cross-platform con le funzioni del DOS (getch())
 * o di un*x (ncurses).
 */
void uiChiediConferma();

/**
 *  Stampa il banner del programma.
 */
void uiStampaBanner();

#endif
